package testing

import (
	"fmt"
	"io/ioutil"
	. "launchpad.net/gocheck"
	"launchpad.net/juju-core/charm"
	"launchpad.net/juju-core/environs"
	"launchpad.net/juju-core/environs/dummy"
	"launchpad.net/juju-core/juju"
	state "launchpad.net/juju-core/state"
	"launchpad.net/juju-core/testing"
	"os"
	"path/filepath"
)

// JujuConnSuite provides a freshly bootstrapped juju.Conn
// for each test. It also includes testing.LoggingSuite.
//
// It also sets up RootDir to point to a directory hierarchy
// mirroring the intended juju directory structure, including
// the following:
// 	RootDir/home/ubuntu/.juju/environments.yaml
//		The dummy environments.yaml file, holding
//		a default environment named "dummyenv"
//		which uses the "dummy" environment type.
//	RootDir/var/lib/juju
//		An empty directory returned as DataDir - the
//		root of the juju data storage space.
// $HOME is set to point to RootDir/home/ubuntu.
type JujuConnSuite struct {
	testing.LoggingSuite
	testing.FakeRootSuite
	testing.MgoSuite
	Conn    *juju.Conn
	State   *state.State
	RootDir string // The faked-up root directory.
	oldHome string
}

// InvalidStateInfo holds information about no state - it will always
// give an error when connected to.  The machine id gives the machine id
// of the machine to be started
func InvalidStateInfo(machineId int) *state.Info {
	return &state.Info{
		Addrs:      []string{"0.1.2.3:1234"},
		EntityName: state.MachineEntityName(machineId),
		Password:   "unimportant",
	}
}

const AdminSecret = "dummy-secret"

var config = []byte(`
environments:
    dummyenv:
        type: dummy
        state-server: true
        authorized-keys: 'i-am-a-key'
        default-series: decrepit
        admin-secret: ` + AdminSecret + `
`)

func (s *JujuConnSuite) SetUpSuite(c *C) {
	s.LoggingSuite.SetUpSuite(c)
	s.FakeRootSuite.SetUpSuite(c)
	s.MgoSuite.SetUpSuite(c)
}

func (s *JujuConnSuite) TearDownSuite(c *C) {
	s.MgoSuite.TearDownSuite(c)
	s.FakeRootSuite.TearDownSuite(c)
	s.LoggingSuite.TearDownSuite(c)
}

func (s *JujuConnSuite) SetUpTest(c *C) {
	s.LoggingSuite.SetUpTest(c)
	s.FakeRootSuite.SetUpTest(c)
	s.MgoSuite.SetUpTest(c)
	s.setUpConn(c)
}

func (s *JujuConnSuite) TearDownTest(c *C) {
	s.tearDownConn(c)
	s.MgoSuite.TearDownTest(c)
	s.FakeRootSuite.TearDownTest(c)
	s.LoggingSuite.TearDownTest(c)
}

// Reset returns environment state to that which existed at the start of
// the test.
func (s *JujuConnSuite) Reset(c *C) {
	s.tearDownConn(c)
	s.FakeRootSuite.Reset(c)
	s.setUpConn(c)
}

func (s *JujuConnSuite) StateInfo(c *C) *state.Info {
	return &state.Info{
		Addrs:    []string{testing.MgoAddr},
		Password: "dummy-secret",
	}
}

func (s *JujuConnSuite) setUpConn(c *C) {
	if s.Conn != nil {
		panic("JujuConnSuite not torn down before being used again")
	}
	err := ioutil.WriteFile(filepath.Join(s.Home, ".juju", "environments.yaml"), config, 0600)
	c.Assert(err, IsNil)

	environ, err := environs.NewFromName("dummyenv")
	c.Assert(err, IsNil)
	// sanity check we've got the correct environment.
	c.Assert(environ.Name(), Equals, "dummyenv")
	c.Assert(environ.Bootstrap(false), IsNil)

	conn, err := juju.NewConnFromName("dummyenv")
	c.Assert(err, IsNil)
	s.Conn = conn
	s.State = conn.State
	c.Assert(err, IsNil)
}

func (s *JujuConnSuite) tearDownConn(c *C) {
	// Bootstrap will set the admin password, and render non-authorized use
	// impossible. s.State may still hold the right password, so try to reset
	// the password so that the MgoSuite soft-resetting works. If that fails,
	// it will still work, but it will take a while since it has to kill the
	// whole database and start over.
	if err := s.State.SetAdminPassword(""); err != nil {
		c.Logf("cannot reset admin password: %v", err)
	}
	c.Assert(s.Conn.Close(), IsNil)
	dummy.Reset()
	s.Conn = nil
	s.State = nil
}

func (s *JujuConnSuite) DataDir() string {
	if s.Conn == nil {
		panic("DataDir called out of test context")
	}
	return filepath.Join(s.RootDir, "/var/lib/juju")
}

// WriteConfig writes a juju config file to the "home" directory.
func (s *JujuConnSuite) WriteConfig(config string) {
	if s.Conn == nil {
		panic("WriteConfig called outside test context")
	}
	path := filepath.Join(os.Getenv("HOME"), ".juju", "environments.yaml")
	err := ioutil.WriteFile(path, []byte(config), 0600)
	if err != nil {
		panic(err)
	}
}

func (s *JujuConnSuite) AddTestingCharm(c *C, name string) *state.Charm {
	ch := testing.Charms.Dir("series", name)
	ident := fmt.Sprintf("%s-%d", ch.Meta().Name, ch.Revision())
	curl := charm.MustParseURL("local:series/" + ident)
	repo, err := charm.InferRepository(curl, testing.Charms.Path)
	c.Assert(err, IsNil)
	sch, err := s.Conn.PutCharm(curl, repo, false)
	c.Assert(err, IsNil)
	return sch
}
